
from ptonppl.control import search
from ptonppl.abstract import AbstractPtonPerson

__author__ = "Jérémie Lumbroso <lumbroso@cs.princeton.edu>"

__version__ = "0.0.3"

__all__ = [
    "search",
    "AbstractPtonPerson",
]




import typing

import ptonppl.abstract
import ptonppl.constants
import ptonppl.ldap
import ptonppl.ldapcmd
import ptonppl.webdir


__author__ = "Jérémie Lumbroso <lumbroso@cs.princeton.edu>"

__all__ = [
    "search",
]


def search(
        value: str,
) -> typing.Optional[ptonppl.abstract.AbstractPtonPerson]:

    obj: typing.Optional[ptonppl.abstract.AbstractPtonPerson] = None

    attempts = []

    if "@" in value:
        # using `old_value` instead of `val` is intentional here
        old_value = value
        attempts += [lambda _: ptonppl.ldap.search_one(ldap_field="mail", ldap_value=old_value)]
        attempts += [lambda _: ptonppl.webdir.search_one(field="mail", value=old_value)]
        attempts += [lambda _: ptonppl.ldapcmd.search_one(ldap_field="mail", ldap_value=old_value)]

        # remove suffix of email, as it might match an alias or a NetID search
        value = value.split("@")[0]

    # operations for NetID and alias

    attempts += [lambda val: ptonppl.ldap.search_one(ldap_field="uid", ldap_value=val)]
    attempts += [lambda val: ptonppl.ldap.search_one(
        ldap_field="mail", ldap_value=ptonppl.constants.WEBDIR_EMAIL_FROM_NETID.format(val))]
    attempts += [lambda val: ptonppl.webdir.search_one(field="uid", value=val)]
    attempts += [lambda val: ptonppl.webdir.search_one(field="alias", value=val)]
    attempts += [lambda val: ptonppl.ldapcmd.search_one(ldap_field="uid", ldap_value=val)]
    attempts += [lambda val: ptonppl.ldapcmd.search_one(
        ldap_field="mail", ldap_value=ptonppl.constants.WEBDIR_EMAIL_FROM_NETID.format(val))]

    # heuristic for PUIDs
    if value.isdigit() and len(value) > 6:
        attempts += [lambda val: ptonppl.ldap.search_one(ldap_field="universityid", ldap_value=val)]
        attempts += [lambda val: ptonppl.ldapcmd.search_one(ldap_field="universityid", ldap_value=val)]

    # reestablish connection
    ptonppl.ldap.connect(reconnect=True)

    for f in attempts:
        try:
            new_obj = f(value)
        except ValueError:
            continue

        if new_obj is None:
            continue

        elif obj is None:
            obj = new_obj

        else:
            obj = obj.merge(obj=new_obj)

        if obj.complete:
            break

    return obj




import distutils.version
import enum
import typing

import ldap


__author__ = "Jérémie Lumbroso <lumbroso@cs.princeton.edu>"

__all__ = [
    "LDAP_HOSTNAME",
    "LDAP_URI",
    "LDAP_24_API",
    "LDAP_BASE_DN",

    "ATTRIBUTES_TOP_LEVEL",
    "ATTRIBUTES_PUBLIC",
    "ATTRIBUTES_FULL",
    "LDAP_ATTRIBUTE_MAPPING"
]


# Princeton LDAP server
LDAP_HOSTNAME: str = "ldap.princeton.edu"

# LDAP URI with protocol scheme
LDAP_URI: str = "ldap://{hostname}".format(hostname=LDAP_HOSTNAME)

# Flag on whether using SSL/TLS
LDAP_URI_SECURE: bool = "ldaps://" in LDAP_URI

# Determine which version of the LDAP package
LDAP_24_API: bool = (distutils.version.LooseVersion(ldap.__version__) >=
                     distutils.version.LooseVersion("2.4"))

# Base domain name for Princeton University
LDAP_BASE_DN: str = "o=Princeton University,c=US"


# Cached sets of attributes

ATTRIBUTES_TOP_LEVEL: typing.Set[str] = {"o", "c"}

ATTRIBUTES_PUBLIC: typing.Set[str] = {
    "cn",
    "displayName",
    "givenName",
    "mail",
    "objectClass",
    "pudisplayname",
    "sn"
}

ATTRIBUTES_FULL: typing.Set[str] = {
    "cn",
    "displayName",
    "eduPersonAffiliation",
    "eduPersonEntitlement",
    "eduPersonPrimaryAffiliation",
    "eduPersonPrincipalName",
    "facsimileTelephoneNumber",
    "givenName",
    "loginShell",
    "mail",
    "objectClass"
    "ou",
    "puacademiclevel",
    "puclassyear",
    "pudisplayname",
    "puhomedepartmentnumber",
    "puinterofficeaddress",
    "purescollege",
    "pustatus",
    "sn",
    "street",
    "telephoneNumber",
    "title",
    "uid",
    "universityid",
    "universityidref",
}

# Mappings with Princeton terminology

LDAP_ATTRIBUTE_MAPPING = {
    "puid": "universityid",
    "netid": "uid",
    "email": "mail",
    "alias": "mail",
    "name": "cn",
}


# Web filter operations

class WebdirFilterOps(enum.Enum):
    CONTAINS = "c"
    IS = "eq"
    IS_NOT = "neq"
    BEGINS_WITH = "b"
    ENDS_WITH = "e"


class WebdirFields(enum.Enum):
    FIRST_NAME = "f"
    LAST_NAME = "l"
    TITLE = "t"
    DEPARTMENT = "d"
    ADDRESS = "a"
    PHONE = "p"
    FAX = "fa"
    NETID = "i"
    EMAIL = "e"


WEBDIR_FIELD_MAPPING = {
    "netid": WebdirFields.NETID,
    "email": WebdirFields.EMAIL,
    "alias": WebdirFields.EMAIL,
}


WEBDIR_EMAIL_FROM_NETID = "{}@princeton.edu"

PRINCETON_CAMPUS_DIRECTORY_SEARCH_URL = \
    "https://www.princeton.edu/search/people-advanced?{field}={{}}&{field}f={filter_op}"


# URL to build queries to search the campus directory by email address equality
# NOTE: for email, had to switch to "begins with" because of bug in search

PRINCETON_CAMPUS_DIRECTORY_EMAIL_SEARCH_URL = PRINCETON_CAMPUS_DIRECTORY_SEARCH_URL.format(
    field=WebdirFields.EMAIL.value,
    filter_op=WebdirFilterOps.BEGINS_WITH.value,
)
PRINCETON_CAMPUS_DIRECTORY_NETID_SEARCH_URL = PRINCETON_CAMPUS_DIRECTORY_SEARCH_URL.format(
    field=WebdirFields.NETID.value,
    filter_op=WebdirFilterOps.IS.value,
)


# LDAP CMD constants

ID_AUTHORIZED_CHARS = "abcdefghijklmnopqrstuvwxyz.0123456789_"

LDAP_IGNORE_FIELDS = ["search", "result"]

LDAP_CMD_PATTERN = "{{cmd}} -x -h {host} -u -b {base_dn} \"{{query}}\"".format(
    host=LDAP_HOSTNAME,
    base_dn=LDAP_BASE_DN,
)

LDAP_DEFAULT_CMD = "ldapsearch"

LDAP_QUERY_NETID = "uid={}"

LDAP_QUERY_EMAIL = "mail={}"

LDAP_QUERY_PUID = "universityid={}"

LDAP_DEFAULT_PROXY_URL = "https://edutools.cs.princeton.edu/integration/ldap.cgi"

PARSED_LDAP_KEY = "uid"


# Output formats
OUTPUT_CSV_HEADER = ["puid", "netid", "email", "alias", "type", "name"]




import typing

import backoff
import ldap

import ptonppl.abstract
import ptonppl.constants


__author__ = "Jérémie Lumbroso <lumbroso@cs.princeton.edu>"

__all__ = [
    "connect",
    "LdapPtonPerson",
    "search_one",
]


# If using LDAPS, ignore server side certificate errors (this assumes
# a self-signed certificate).

if ptonppl.constants.LDAP_URI_SECURE:
    ldap.set_option(ldap.OPT_X_TLS_REQUIRE_CERT, ldap.OPT_X_TLS_ALLOW)

# Don't follow referrals

ldap.set_option(ldap.OPT_REFERRALS, 0)

# Short time out (both for initial connection and follow-up operations)

ldap.set_option(ldap.OPT_NETWORK_TIMEOUT, 2)
ldap.set_option(ldap.OPT_TIMEOUT, 1)

# Connect to server

_server: typing.Optional[ldap.ldapobject.LDAPObject] = None


def connect(
        reconnect: typing.Optional[bool] = None,
) -> ldap.ldapobject.LDAPObject:
    global _server

    if _server is None or (reconnect is not None and reconnect):
        _server = ldap.initialize(ptonppl.constants.LDAP_URI)
        _server.protocol_version = 3

    return _server


connect(reconnect=True)


# Determine attributes that are available (on a sample of 100 results)

def get_attributes() -> typing.Set[str]:
    msg_id = _server.search_ext(
        ptonppl.constants.LDAP_BASE_DN,
        ldap.SCOPE_SUBTREE,
        sizelimit=100,
    )

    fields = set()

    while True:
        try:
            _, new_results = _server.result(msgid=msg_id, all=0)
        except ldap.LDAPError as error:
            break

        if new_results is None or type(new_results) is not list:
            break

        try:
            new_result = new_results[0]
            _, new_record = new_result
        except ValueError as error:
            break

        fields = fields.union(set(new_record.keys()))

    return fields


_ldap_attributes: typing.Set[str] = get_attributes()


# If we only have a small number of attributes in common with ATTRIBUTES_FULL
# we are probably in restricted LDAP

_ldap_restricted: bool = (
        len(_ldap_attributes.intersection(ptonppl.constants.ATTRIBUTES_FULL)) <=
        len(ptonppl.constants.ATTRIBUTES_PUBLIC)
)


def _ldap_do_we_give_up(
        err: typing.Union[ldap.SERVER_DOWN, ldap.TIMEOUT, Exception]
) -> bool:

    if isinstance(err, ldap.SERVER_DOWN) or isinstance(err, ldap.TIMEOUT):
        # reconnect ...
        connect(reconnect=True)

        # ... and retry
        return False

    # neither one of those exceptions, therefore we should fail
    return True


ldap_retry = backoff.on_exception(
    wait_gen=backoff.constant,
    exception=ldap.LDAPError,
    giveup=_ldap_do_we_give_up
)


@ldap_retry
def search_aux(
        ldap_filter: typing.Optional[str] = None,
        size_limit: int = 1,
):
    msg_id = _server.search_ext(
        base=ptonppl.constants.LDAP_BASE_DN,
        scope=ldap.SCOPE_SUBTREE,
        filterstr=ldap_filter,
        sizelimit=size_limit,
    )

    results = []

    while True:
        try:
            _, new_results = _server.result(msgid=msg_id, all=0)
            if new_results is None or len(new_results) == 0:
                break

            new_result = new_results[0]
            res_type, res_data = new_result

            aux_attrs = {
                field: value
                for (field, value) in map(lambda s: s.split("="), map(str.strip, res_type.split(",")))
                if field.lower() not in ptonppl.constants.ATTRIBUTES_TOP_LEVEL
            }
            res_data.update(aux_attrs)

            results.append(res_data)

        except ldap.SIZELIMIT_EXCEEDED:
            break

        except ValueError:
            continue

    return results


def search_many(
        ldap_field: str,
        ldap_value: str,
        size_limit: int = 1,
):
    if _ldap_restricted and ldap_field not in ptonppl.constants.ATTRIBUTES_PUBLIC:
        return None

    return search_aux(
        ldap_filter="{}={}".format(ldap_field, ldap_value),
        size_limit=size_limit,
    )


def _grab_attr(
        ldap_obj: typing.Dict[str, typing.Any],
        ldap_field: str
) -> typing.Optional[typing.Any]:

    if ldap_obj is None:
        return

    val = ldap_obj.get(ldap_field)
    if val is None:
        return

    if type(val) is list:
        try:
            val = val[0]
        except ValueError:
            pass

    if type(val) is bytes:
        val = val.decode("ascii")

    return val


class LdapPtonPerson(ptonppl.abstract.AbstractPtonPerson):

    def __init__(self, ldap_result: typing.Dict[str, typing.Any]):
        self._original = ldap_result

        self._puid = _grab_attr(self._original, ptonppl.constants.LDAP_ATTRIBUTE_MAPPING["puid"])
        self._netid = _grab_attr(self._original, ptonppl.constants.LDAP_ATTRIBUTE_MAPPING["netid"])
        self._email = _grab_attr(self._original, ptonppl.constants.LDAP_ATTRIBUTE_MAPPING["email"])
        self._alias = _grab_attr(self._original, ptonppl.constants.LDAP_ATTRIBUTE_MAPPING["alias"])
        self._pustatus = _grab_attr(self._original, "pustatus")
        self._cn = _grab_attr(self._original, ptonppl.constants.LDAP_ATTRIBUTE_MAPPING["name"])

        if self._email is None:
            val = _grab_attr(self._original, "eduPersonPrincipalName")
            if "@" in val:
                self._email = val

        if self._alias is not None:
            self._alias = self._alias.split("@")[0]


def search_one(
        ldap_field: str,
        ldap_value: str,
) -> typing.Optional[LdapPtonPerson]:

    ret = search_many(
        ldap_field=ldap_field,
        ldap_value=ldap_value,
        size_limit=1,
    )

    if ret is not None and len(ret) > 0:
        return LdapPtonPerson(ldap_result=ret[0])



import os
import io
import sys
import textwrap
import time
import typing

import click
import click_help_colors

import ptonppl
import ptonppl.abstract
import ptonppl.constants
import ptonppl.control


cli_opt_version = click.version_option(version=ptonppl.__version__)

OutputFormatType = typing.Union[
    typing.Literal["term"],
    typing.Literal["json"],
    typing.Literal["csv"],
]

cli_opt_output_format = click.option(
    "--format", "-f",
    type=click.Choice(["term", "json", "csv"], case_sensitive=False),
    default="term", metavar="FORMAT",
    help="Output format (e.g.: term, json, csv, ...)"
)


@click.command(
    cls=click_help_colors.HelpColorsCommand,
    help_headers_color='yellow',
    help_options_color='bright_yellow'
)
@click.argument("query", nargs=-1, required=True)
@cli_opt_output_format
@click.option(
    "--unique", "-u",
    is_flag=True, default=True,
    help="Filter out duplicate records from the output."
)
@cli_opt_version
def cli(
        query: typing.Tuple[str],
        format: OutputFormatType,
        unique: bool,
):
    """
    Lookup the directory information (PUID, NetID, email, name) of any
    Princeton campus person, using whichever of LDAP, web directory or
    proxy server is available.
    """

    # Initial statistics

    count_total: int = len(query)
    count_success: int = 0
    count_error: int = 0
    count_duplicate: int = 0

    time_start: float = time.time()

    # NetIDs should be a unique, and fairly available piece of data to
    # check whether we've seen an object before

    seen_netids: typing.Set[str] = set()
    results: typing.List[ptonppl.abstract.AbstractPtonPerson] = list()

    if format == "json":
        print("{")

    elif format == "csv":
        print(",".join(ptonppl.constants.OUTPUT_CSV_HEADER))

    for q in query:

        # lookup object
        obj = ptonppl.control.search(value=q)

        # may not have been found
        if obj is None:
            count_error += 1
            continue

        count_success += 1

        if obj.netid in seen_netids:
            count_duplicate += 1
            if unique:
                continue

        # save new results
        seen_netids.add(obj.netid)
        results.append(obj)

        if format == "json":
            print("    {},".format(obj.as_dict.__repr__()))

        elif format == "csv":
            print(",".join(
                list(map(
                    lambda field: '"{}"'.format(obj.as_dict.get(field, "").replace('"', '\"')),
                    ptonppl.constants.OUTPUT_CSV_HEADER))))

        elif format == "term":
            print(obj.common_name)
            print("  {}".format(obj.puid))
            print("  {}".format(obj.netid))
            print("  {}".format(obj.email))
            print()

    if format == "json":
        print("}")

    # End statistics

    count_results = len(results)
    time_end: float = time.time()
    time_elapsed: float = time_end - time_start


def main():
    return sys.exit(cli())


if __name__ == "__main__":
    main()




import typing

import requests
import bs4

import ptonppl.constants
import ptonppl.ldap


__author__ = "Jérémie Lumbroso <lumbroso@cs.princeton.edu>"

__all__ = [
    "search_one"
]


# Hard-coded constants that are required to scrape the web page

CLASS_RESULTS_BLOCK1 = "people-results"
CLASS_RESULTS_BLOCK2 = "bordered"
CLASS_RESULTS_ROW = "row"
CLASS_RESULTS_DETAILS = "expanded-details-value"


# noinspection PyBroadException
def _fetch_raw_results(url: str) -> typing.Optional[typing.List[bs4.element.Tag]]:

    # not a valid URL
    if url is None or type(url) is not str or len(url) == 0 or "http" not in url:
        return

    # make request
    r = requests.get(url)
    if not r.ok:
        return

    # parse using BeautifulSoup
    s = bs4.BeautifulSoup(r.content, features="html.parser")
    if s is None:
        return

    # try to extract the subtree of results
    try:
        # get the subtree of the DOM containing results
        b = s.find("div", attrs={"class": CLASS_RESULTS_BLOCK1}).find("div", attrs={"class": CLASS_RESULTS_BLOCK2})

        # get individual results
        raw_items = b.find_all("div", attrs={"class": CLASS_RESULTS_ROW})

        # ensure there are least one
        if raw_items is None:
            return

    except:
        return

    return raw_items


def _get_inline_content(
        obj: bs4.element.Tag,
        class_name: str
) -> typing.Optional[str]:
    obj = obj.find(attrs={"class": class_name})
    if obj is None:
        return

    str_value = "".join(obj.stripped_strings) or ""
    str_value = str_value.strip()

    return str_value


def _get_header_content(
        obj: bs4.element.Tag,
        header_name: str
) -> typing.Optional[str]:
    tag_label = obj.find("h4", string=header_name)
    if tag_label is None:
        return

    tag_value = tag_label.find_next_sibling("span", attrs={"class": CLASS_RESULTS_DETAILS})
    if tag_value is None:
        return

    str_value = "".join(tag_value.stripped_strings) or ""
    str_value = str_value.strip()

    return str_value


def _parse_result(obj: bs4.element.Tag) -> typing.Dict[str, typing.Any]:

    r = dict()

    # NOTE: should remove all the hardcoded constants from below perhaps?
    # or move to constants module

    for header_name, field_name in [
        ("NetID", "uid"),
        ("University ID", "universityid"),
        ("Office Location", "street"),
        ("Interoffice Address", "puinterofficeaddress"),
    ]:
        val = _get_header_content(obj=obj, header_name=header_name)
        if val is not None:
            r[field_name] = val

    for class_name, field_name in [
        ("title", "title"),
        ("people-search-email", "mail"),
        ("people-search-result-phone", "telephoneNumber"),
        ("people-search-result-name", "pudisplayname"),
        ("people-search-result-department", "ou"),
    ]:
        val = _get_inline_content(obj=obj, class_name=class_name)
        if val is not None:
            r[field_name] = val

    if "pudisplayname" in r:
        pudn = r.get("pudisplayname")
        last, first = pudn.split(", ")
        r["displayName"] = "{} {}".format(first, last)
        r["givenName"] = first
        r["cn"] = r["displayName"]
        r["sn"] = last

    if "mail" in r and "uid" in r:
        r["eduPersonPrincipalName"] = ptonppl.constants.WEBDIR_EMAIL_FROM_NETID.format(r["uid"])

    return r


def search_one(
        field: str,
        value: str,
) -> typing.Optional[ptonppl.ldap.LdapPtonPerson]:

    if field.lower() in ["mail", "email"]:
        url_pattern = ptonppl.constants.PRINCETON_CAMPUS_DIRECTORY_EMAIL_SEARCH_URL

    elif field.lower() in ["alias"]:
        url_pattern = ptonppl.constants.PRINCETON_CAMPUS_DIRECTORY_EMAIL_SEARCH_URL
        value = ptonppl.constants.WEBDIR_EMAIL_FROM_NETID.format(value)

    elif field.lower() in ["netid"]:
        url_pattern = ptonppl.constants.PRINCETON_CAMPUS_DIRECTORY_NETID_SEARCH_URL

    else:
        raise ValueError(
            "unrecognized field: {}  (choices are 'mail', 'uid')".format(field))

    url = url_pattern.format(value)

    raw_results = _fetch_raw_results(url=url)
    if raw_results is None:
        return

    results = list(map(_parse_result, raw_results))

    if results is not None and len(results) > 0:
        return ptonppl.ldap.LdapPtonPerson(ldap_result=results[0])




import copy
import typing


__author__ = "Jérémie Lumbroso <lumbroso@cs.princeton.edu>"

__all__ = [
    "AbstractPtonPerson",
]


class AbstractPtonPerson:

    _puid: typing.Optional[int] = None
    _netid: typing.Optional[str] = None
    _alias: typing.Optional[str] = None
    _email: typing.Optional[str] = None
    _pustatus: typing.Optional[str] = None
    _cn: typing.Optional[str] = None

    _original: typing.Optional[typing.Any] = None

    def merge(self, obj: 'AbstractPtonPerson') -> 'AbstractPtonPerson':
        # defensive copy
        self_copy = copy.deepcopy(self)

        # pick best version of attribute
        self_copy._puid = self_copy._puid or obj._puid
        self_copy._netid = self_copy._netid or obj._netid
        self_copy._alias = self_copy._alias or obj._alias
        self_copy._email = self_copy._email or obj._email
        self_copy._pustatus = self_copy._pustatus or obj._pustatus
        self_copy._cn = self_copy._cn or obj._cn

        # update original dict
        if obj._original is not None:
            orig = copy.deepcopy(obj._original)
            if self_copy._original is not None:
                orig.update(self_copy._original)
            self_copy._original = orig

        return self_copy

    @property
    def complete(self):
        return (
                self._puid is not None and
                self._netid is not None and
                self._email is not None
        )

    @property
    def puid(self) -> typing.Optional[int]:
        return self._puid

    @property
    def netid(self) -> typing.Optional[str]:
        return self._netid

    @property
    def alias(self) -> typing.Optional[str]:
        return self._alias

    @property
    def email(self) -> typing.Optional[str]:
        return self._email

    @property
    def common_name(self) -> typing.Optional[str]:
        return self._cn

    @property
    def status(self) -> typing.Optional[str]:
        return self._pustatus

    @property
    def has_alias(self) -> bool:
        return self._alias is not None and self._alias != self._netid

    @property
    def original(self) -> typing.Any:
        # defensive copy
        if self._original is not None:
            return copy.deepcopy(self._original)

    @property
    def as_dict(self) -> typing.Dict[str, typing.Any]:
        ret = dict()

        for (name, val) in [
            ("puid", self._puid),
            ("netid", self._netid),
            ("alias", self._alias),
            ("email", self._email),
            ("type", self._pustatus),
            ("name", self._cn),
        ]:
            if val is not None:
                ret[name] = val

        if "alias" in ret and "netid" in ret and ret["alias"] == ret["netid"]:
            del ret["alias"]

        return ret



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['ptonppl']

package_data = \
{'': ['*']}

install_requires = \
['backoff>=1.10.0,<2.0.0',
 'bs4>=0.0.1,<0.0.2',
 'click-option-group>=0.5.1,<0.6.0',
 'click>=7.1.2,<8.0.0',
 'click_help_colors>=0.8,<0.9',
 'python-ldap>=3.3.1,<4.0.0',
 'requests>=2.24.0,<3.0.0']

entry_points = \
{'console_scripts': ['ptonppl = ptonppl.__main__:main']}

setup_kwargs = {
    'name': 'ptonppl',
    'version': '0.0.3',
    'description': 'An integration package to lookup Princeton campus users.',
    'long_description': '# ptonppl — Princeton People\n\nThis provides a Python package and a command-line tool to lookup the\ncampus directory of any member of the Princeton community. The package\nprovides a unified search function that queries the following fields:\n- PUID, *e.g.*, `902312554`\n- NetID, *e.g.*, `lumbroso`\n- Alias, when the user has defined one, *e.g.*, `jeremie.lumbroso`\n- Email, *e.g.*, `lumbroso@princeton.edu`\n\nThis information is hard to come by consistently, and this tool seeks\nto provide a robust interface to the information.\n\n## Installation\n\nThe package is distributed on PyPI and can be installed using the usual\ntools, such as `pip` or `pipenv`:\n```shell\n$ pip install --user ptonppl\n```\n\n## Help message\n\n```\n$ ptonppl --help\n\nUsage: ptonppl [OPTIONS] QUERY...\n\n  Lookup the directory information (PUID, NetID, email, name) of any\n  Princeton campus person, using whichever of LDAP, web directory or proxy\n  server is available.\n\nOptions:\n  -f, --format FORMAT  Output format (e.g.: term, json, csv, ...)\n  -u, --unique         Filter out duplicate records from the output.\n  --version            Show the version and exit.\n  --help               Show this message and exit.\n```\n\n## License\n\nThis project is licensed under the LGPLv3 license, with the understanding\nthat importing a Python modular is similar in spirit to dynamically linking\nagainst it.\n\n- You can use the library `ptonppl` in any project, for any purpose, as long\n  as you provide some acknowledgement to this original project for use of\n  the library.\n\n- If you make improvements to `ptonppl`, you are required to make those\n  changes publicly available.',
    'author': 'Jérémie Lumbroso',
    'author_email': 'lumbroso@cs.princeton.edu',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/jlumbroso/ptonppl',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)




import typing
import subprocess

import requests

import ptonppl.abstract
import ptonppl.constants
import ptonppl.ldap


__author__ = "Jérémie Lumbroso <lumbroso@cs.princeton.edu>"

__all__ = [
    "search_one",
]


def _check_uid(s:str) -> typing.Optional[str]:
    if not (s or type(s) is str):
        return None

    s_l = s.lower()
    for c in s_l:
        if c not in ptonppl.constants.ID_AUTHORIZED_CHARS:
            return None

    return s_l


def _parse_ldapsearch_output(out:str) -> typing.Dict[str, typing.Dict[str, str]]:

    if out is None:
        return dict()

    lines = out.split("\n")

    # sentinel to flush
    lines.append("dn: ")

    current_dn = None
    current_record = {}

    identities = {}

    for line in lines:
        if line == "" or line[0] == "#":
            continue

        if ":" in line:
            (field, value) = line.split(":", 1)
            field = field.strip().lower()
            value = value.strip().lower()

            if field == "dn":

                # the last occurrence of "dn" is a sentinel meant to flush
                # the final result

                if current_dn is not None and current_record is not None:
                    key = current_record[ptonppl.constants.PARSED_LDAP_KEY]
                    identities[key] = current_record

                if value:
                    current_dn = value
                    current_record = {}

            elif field:

                if field not in ptonppl.constants.LDAP_IGNORE_FIELDS:

                    if field in current_record:
                        curr = current_record[field]

                        if type(curr) is list:
                            curr.append(value)
                        else:
                            current_record[field] = [curr, value]

                    else:
                        current_record[field] = value

    return identities


def _get_ldapsearch_output_from_proxy_url(
        ldap_field: str,
        ldap_value: str,
        proxy_url: typing.Optional[str] = None,
) -> typing.Optional[str]:

    if proxy_url is None:
        proxy_url = ptonppl.constants.LDAP_DEFAULT_PROXY_URL

    r = requests.get(proxy_url, params={ldap_field: ldap_value})

    if r.ok:
        return r.content.decode("ascii")


def _get_ldapsearch_output_from_local_cmd(
        ldap_field: str,
        ldap_value: str,
        local_cmd: typing.Optional[str] = None,
) -> typing.Optional[str]:

    if local_cmd is None:
        local_cmd = ptonppl.constants.LDAP_DEFAULT_CMD

    cmd = ptonppl.constants.LDAP_CMD_PATTERN.format(
        cmd=local_cmd,
        query="{}={}".format(ldap_field, ldap_value),
    )

    (status, out) = subprocess.getstatusoutput(cmd)

    # If unsuspected problem, check here
    # FIXME: add error correction
    ## TO DEBUG:
    ## print "Content-type: text/plain\n\n", (status, output)
    ## if error 256 may be that the local copy of ldapsearch is outdated

    if status == 0:
        return out


def _get_ldapsearch_output(
        ldap_field: str,
        ldap_value: str,
) -> typing.Optional[typing.Dict[str, str]]:

    # escape bad characters
    ldap_value = _check_uid(ldap_value)
    if ldap_value is None:
        return

    local_ret = _parse_ldapsearch_output(
        out=_get_ldapsearch_output_from_local_cmd(
            ldap_field=ldap_field,
            ldap_value=ldap_value,
        )
    )

    url_ret = _parse_ldapsearch_output(
        out=_get_ldapsearch_output_from_proxy_url(
            ldap_field=ldap_field,
            ldap_value=ldap_value,
        )
    )

    obj1 = list(local_ret.values())[0] if len(local_ret) > 0 else None
    obj2 = list(url_ret.values())[0] if len(url_ret) > 0 else None

    if obj1 is None:
        return obj2

    if obj1 is not None and obj2 is not None:
        obj1.update(obj2)

    return obj1


def search_one(
        ldap_field: str,
        ldap_value: str,
) -> typing.Optional[ptonppl.abstract.AbstractPtonPerson]:

    ret = _get_ldapsearch_output(
        ldap_field=ldap_field,
        ldap_value=ldap_value,
    )

    if ret is not None:
        return ptonppl.ldap.LdapPtonPerson(ldap_result=ret)
